@@ -84,6 +84,28 @@ struct CDDrivePriv {
 
 static CDDrive *cd_drive_new (void);
 
+#ifdef USE_HAL
+static LibHalContext *
+get_hal_context (void)
+{
+	static LibHalContext *ctx = NULL;
+	LibHalFunctions hal_functions = {
+		NULL, /* mainloop integration */
+		NULL, /* device_added */
+		NULL, /* device_removed */
+		NULL, /* device_new_capability */
+		NULL, /* property_modified */
+		NULL, /* device_condition */
+	};
+	
+	if (ctx == NULL)
+		ctx = hal_initialize (&hal_functions, FALSE);
+
+	return ctx;
+}
+#endif /* USE_HAL */
+
+
 /* Utility functions, be careful to have a match with what's use in the
  * different bits of code */
 #if !defined (__linux__)
@@ -196,6 +218,7 @@ int get_mmc_profile (int fd);
 int get_disc_size_cd (int fd);
 gint64 get_disc_size_dvd (int fd, int mmc_profile);
 int get_read_write_speed (int fd, int *read_speed, int *write_speed);
+int get_disc_status (int fd, int *empty, int *is_rewritable);
 
 
 static void
@@ -227,7 +250,6 @@ cd_drive_door_open (gboolean mmc_profile, int fd)
 		int status;
 
 		status = ioctl (fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
-		close (fd);
 		if (status < 0) {
 			return FALSE;
 		}
@@ -243,16 +265,14 @@ cd_drive_door_open (gboolean mmc_profile, int fd)
 #endif
 }
 
-CDMediaType
-cd_drive_get_media_type_from_path (const char *device)
+static CDMediaType
+cd_drive_get_media_type_from_path_full (const char *device, gboolean *is_rewritable)
 {
 	int fd;
 	int mmc_profile;
 
 	g_return_val_if_fail (device != NULL, CD_MEDIA_TYPE_ERROR);
 
-	/* FIXME get media type from HAL if it's there BZ 145196 */
-
 	fd = open (device, O_RDONLY|O_EXCL|O_NONBLOCK);
 	if (fd < 0) {
 		if (errno == EBUSY) {
@@ -268,11 +288,21 @@ cd_drive_get_media_type_from_path (const char *device)
 		gboolean opened;
 
 		opened = cd_drive_door_open (mmc_profile, fd);
-		close (fd);
 
 		if (opened != FALSE) {
+			close (fd);
 			return CD_MEDIA_TYPE_ERROR;
 		} else {
+			int rewrite, empty;
+			if (get_disc_status (fd, &empty, &rewrite) == 0) {
+				close (fd);
+				*is_rewritable = rewrite;
+				if (empty)
+					return CD_MEDIA_TYPE_ERROR;
+				else
+					return CD_MEDIA_TYPE_UNKNOWN;
+			}
+			close (fd);
 			return CD_MEDIA_TYPE_UNKNOWN;
 		}
 	}
@@ -289,6 +319,7 @@ cd_drive_get_media_type_from_path (const char *device)
 	case 0x9:	/* CD-R                         */
 		return CD_MEDIA_TYPE_CDR;
 	case 0xa:	/* CD-RW			*/
+		*is_rewritable = TRUE;
 		return CD_MEDIA_TYPE_CDRW;
 	case 0x10:	/* Commercial DVDs		*/
 		return CD_MEDIA_TYPE_DVD;
@@ -296,10 +327,12 @@ cd_drive_get_media_type_from_path (const char *device)
 		return CD_MEDIA_TYPE_DVDR;
 	case 0x13:      /* DVD-RW Restricted Overwrite  */
 	case 0x14:      /* DVD-RW Sequential            */
+		*is_rewritable = TRUE;
 		return CD_MEDIA_TYPE_DVDRW;
 	case 0x1B:      /* DVD+R                        */
 		return CD_MEDIA_TYPE_DVD_PLUS_R;
 	case 0x1A:      /* DVD+RW                       */
+		*is_rewritable = TRUE;
 		return CD_MEDIA_TYPE_DVD_PLUS_RW;
 	case 0x12:      /* DVD-RAM                      */
 		return CD_MEDIA_TYPE_DVD_RAM;
@@ -308,12 +341,98 @@ cd_drive_get_media_type_from_path (const char *device)
 	}
 }
 
+
+CDMediaType
+cd_drive_get_media_type_from_path (const char *device)
+{
+	gboolean is_rewritable;
+	
+	return cd_drive_get_media_type_from_path_full (device, &is_rewritable);
+}
+
+
 CDMediaType
 cd_drive_get_media_type (CDDrive *cdrom)
+{
+	gboolean is_rewritable;
+	
+	return cd_drive_get_media_type_and_rewritable (cdrom, &is_rewritable);
+}
+
+CDMediaType
+cd_drive_get_media_type_and_rewritable (CDDrive *cdrom, gboolean *is_rewritable)
 {
 	g_return_val_if_fail (cdrom != NULL, CD_MEDIA_TYPE_ERROR);
 
-	return cd_drive_get_media_type_from_path (cdrom->device);
+	*is_rewritable = FALSE;
+
+#ifdef USE_HAL
+	if (cdrom->priv != NULL && cdrom->priv->udi != NULL) {
+		LibHalContext *ctx;
+		char** device_names;
+		int num_devices;
+		CDMediaType type;
+		char *hal_type;
+		
+		ctx = get_hal_context ();
+		if (ctx != NULL) {
+			device_names = hal_manager_find_device_string_match (ctx, 
+									     "info.parent",
+									     cdrom->priv->udi,
+									     &num_devices);
+			if (num_devices == 0) {
+				return CD_MEDIA_TYPE_ERROR;
+			}
+
+			/* just look at the first child */
+			if (hal_device_get_property_bool (ctx, 
+							  device_names[0],
+							  "volume.is_mounted")) {
+				type = CD_MEDIA_TYPE_BUSY;
+			} else {		    
+				*is_rewritable = hal_device_get_property_bool (ctx, 
+									       device_names[0],
+									       "volume.disc.is_rewritable");
+				type = CD_MEDIA_TYPE_BUSY;
+				hal_type = hal_device_get_property_string (ctx, 
+									   device_names[0],
+									   "volume.disc.type");
+				if (hal_type == NULL || strcmp (hal_type, "unknown") == 0) {
+					type = CD_MEDIA_TYPE_UNKNOWN;
+				} else if (strcmp (hal_type, "cd_rom") == 0) {
+					type = CD_MEDIA_TYPE_CD;
+				} else if (strcmp (hal_type, "cd_r") == 0) {
+					type = CD_MEDIA_TYPE_CDR;
+				} else if (strcmp (hal_type, "cd_rw") == 0) {
+					type = CD_MEDIA_TYPE_CDRW;
+				} else if (strcmp (hal_type, "dvd_rom") == 0) {
+					type = CD_MEDIA_TYPE_DVD;
+				} else if (strcmp (hal_type, "dvd_r") == 0) {
+					type = CD_MEDIA_TYPE_DVDR;
+				} else if (strcmp (hal_type, "dvd_ram") == 0) {
+					type = CD_MEDIA_TYPE_DVD_RAM;
+				} else if (strcmp (hal_type, "dvd_rw") == 0) {
+					type = CD_MEDIA_TYPE_DVDRW;
+				} else if (strcmp (hal_type, "dvd_plus_rw") == 0) {
+					type = CD_MEDIA_TYPE_DVD_PLUS_RW;
+				} else if (strcmp (hal_type, "dvd_plus_r") == 0) {
+					type = CD_MEDIA_TYPE_DVD_PLUS_R;
+				} else {
+					type = CD_MEDIA_TYPE_UNKNOWN;
+				}
+				
+				if (hal_type != NULL)
+					hal_free_string (hal_type);
+			}
+
+			hal_free_string_array (device_names);
+
+			return type;
+		}
+	}
+#endif
+
+	return cd_drive_get_media_type_from_path_full (cdrom->device, is_rewritable);
 }
 
 gint64
@@ -378,20 +497,12 @@ static GList *
 hal_scan (gboolean recorder_only)
 {
 	GList *cdroms = NULL;
-	LibHalFunctions hal_functions = {
-		NULL, /* mainloop integration */
-		NULL, /* device_added */
-		NULL, /* device_removed */
-		NULL, /* device_new_capability */
-		NULL, /* property_modified */
-		NULL, /* device_condition */
-	};
 	int i;
 	int num_devices;
 	char** device_names;
 	LibHalContext *ctx;
 
-	ctx = hal_initialize (&hal_functions, FALSE);
+	ctx = get_hal_context ();
 	if (ctx == NULL) {
 		return NULL;
 	}
@@ -401,7 +512,6 @@ hal_scan (gboolean recorder_only)
 
 	if (device_names == NULL)
 	{
-		hal_shutdown (ctx);
 		return NULL;
 	}
 
@@ -476,8 +586,7 @@ hal_scan (gboolean recorder_only)
 		cdrom->priv->udi = g_strdup (device_names[i]);
 	}
 
-	g_strfreev (device_names);
-	hal_shutdown (ctx);
+	hal_free_string_array (device_names);
 
 	cdroms = g_list_reverse (cdroms);
 
@@ -1355,6 +1464,61 @@ cd_drive_free (CDDrive *drive)
 	g_free (drive);
 }
 
+gboolean
+cd_drive_lock (CDDrive *drive,
+	       const char *reason,
+	       char **reason_for_failure) 
+{
+	gboolean res;
+
+	if (reason_for_failure != NULL)
+		*reason_for_failure = NULL;
+
+	res = TRUE;
+#ifdef USE_HAL
+	if (drive->priv->udi != NULL) {
+		LibHalContext *ctx;
+		char *dbus_reason;
+		
+		ctx = get_hal_context ();
+		if (ctx != NULL) {
+			res = hal_device_lock (ctx, 
+					       drive->priv->udi,
+					       reason,
+					       &dbus_reason);
+			if (dbus_reason != NULL && 
+			    reason_for_failure != NULL)
+				*reason_for_failure = g_strdup (dbus_reason);
+			if (dbus_reason != NULL)
+				dbus_free (dbus_reason);
+		}
+	}
+#endif
+	return res;
+}
+
+
+gboolean 
+cd_drive_unlock (CDDrive *drive)
+{
+	gboolean res;
+
+	res = TRUE;
+#ifdef USE_HAL
+	if (drive->priv->udi != NULL) {
+		LibHalContext *ctx;
+
+		ctx = get_hal_context ();
+		if (ctx != NULL) {
+			res = hal_device_unlock (ctx, 
+						 drive->priv->udi);
+		}
+	}
+#endif
+	return res;
+}
+
+
 static CDDrive *
 cd_drive_new (void)
 {
