@@ -52,7 +52,7 @@ drive_type (int type)
 }
 
 static const char*
-media_type (int type)
+media_type_get_string (int type)
 {
 	switch (type) {
 	case CD_MEDIA_TYPE_BUSY:
@@ -96,19 +96,27 @@ list_cdroms (void)
 	{
 		char *type_str;
 		const char *media;
+		int media_type;
+		gboolean is_rewritable;
 		gint64 size;
 
 		cd = l->data;
 		type_str = drive_type (cd->type);
-		media = media_type (cd_drive_get_media_type (cd));
+		media_type = cd_drive_get_media_type_and_rewritable (cd, &is_rewritable);
+		media = media_type_get_string (media_type);
 
 		g_print ("name: %s device: %s max_speed_read: %d\n",
 				cd->display_name, cd->device,
 				cd->max_speed_read);
 		g_print ("type: %s\n", type_str);
-		g_print ("media type: %s\n", media);
+		g_print ("media type: %s%s\n", media, is_rewritable?" (rewritable)":"");
 		size = cd_drive_get_media_size (cd);
-		g_print ("media size: %0.2f megs (approx. or %d mins %d secs)\n", (float) size / 1024 / 1024, SIZE_TO_TIME(size) / 60, SIZE_TO_TIME(size) % 60);
+		g_print ("media size: %0.2f megs", (float) size / 1024 / 1024);
+		if (media_type == CD_MEDIA_TYPE_CDR || media_type == CD_MEDIA_TYPE_CDRW) {
+			g_print ("approx. or %d mins %d secs)\n", SIZE_TO_TIME(size) / 60, SIZE_TO_TIME(size) % 60);
+		} else {
+			g_print ("\n");
+		}
 		g_print ("CD-Recorder/SCSI devices only: max_speed_write: %d"
 				" id: %s\n", cd->max_speed_write,
 				cd->cdrecord_id);
