@@ -280,30 +280,30 @@ action_changed_cb (CDRecorder *cdrecorder, CDRecorderActions action,
 	switch (action) {
 	case PREPARING_WRITE:
 		if (media == MEDIA_CD) {
-			text = N_("Preparing to write CD");
+			text = _("Preparing to write CD");
 		} else {
-			text = N_("Preparing to write DVD");
+			text = _("Preparing to write DVD");
 		}
 		break;
 	case WRITING:
 		if (media == MEDIA_CD) {
-			text = N_("Writing CD");
+			text = _("Writing CD");
 		} else {
-			text = N_("Writing DVD");
+			text = _("Writing DVD");
 		}
 		break;
 	case FIXATING:
 		if (media == MEDIA_CD) {
-			text = N_("Fixating CD");
+			text = _("Fixating CD");
 		} else {
-			text = N_("Fixating DVD");
+			text = _("Fixating DVD");
 		}
 		break;
 	case BLANKING:
 		if (media == MEDIA_CD) {
-			text = N_("Erasing CD");
+			text = _("Erasing CD");
 		} else {
-			text = N_("Erasing DVD");
+			text = _("Erasing DVD");
 		}
 		break;
 	default:
@@ -331,10 +331,10 @@ insert_cd_request_cb (CDRecorder *cdrecorder,
 	if (busy_cd) {
 		msg = N_("Please make sure another application is not using the disc.");
 		title = N_("Disc is busy");
-	} else if (is_reload && can_rewrite) {
+	} else if (!is_reload && can_rewrite) {
 		msg = N_("Please insert a rewritable or blank media in the drive tray.");
 		title = N_("Insert rewritable or blank media");
-	} else if (is_reload && !can_rewrite) {
+	} else if (!is_reload && !can_rewrite) {
 		msg = N_("Please insert a blank media in the drive tray.");
 		title = N_("Insert blank media");
 	} else if (can_rewrite) {
